'use strict';
const Generator = require('yeoman-generator');
const chalk = require('chalk');
const yosay = require('yosay');
const slug = require('slug');

module.exports = class extends Generator {
  initializing() {
    // Use license generator
    this.composeWith(require.resolve('generator-license'), {
      name: 'In The Pocket',
      email: 'info@inthepocket.mobi',
      website: 'https://inthepocket.mobi',
      licensePrompt: 'Which license do you want to use?',
      defaultLicense: 'MIT',
    });
  }

  prompting() {
    // Have Yeoman greet the user.
    this.log(yosay(
      'Welcome to the ' + chalk.blue('ITP React project') + ' generator!'
    ));

    const prompts = [];

    // Project name
    prompts.push({
      type: 'input',
      name: 'name',
      message: 'Project name?',
      default: slug(this.appname),
    });

    // Project description
    prompts.push({
      type: 'input',
      name: 'desc',
      message: 'Project description?',
      default: this.appname,
    });

    // Repository
    prompts.push({
      type: 'input',
      name: 'repo',
      message: 'Repository url?',
      default: 'https://bitbucket.org/inthepocket/',
    });

    // Create new directory
    prompts.push({
      type: 'confirm',
      name: 'createDir',
      message: 'Create a new directory for your project?',
      default: true,
    });

    // Testing
    prompts.push({
      type: 'list',
      name: 'testing',
      message: 'How do you want to test?',
      choices: [
        'jest',
        'jest+enzyme',
      ],
    });

    // Enable experimental features
    prompts.push({
      type: 'confirm',
      name: 'experimental',
      message: 'Enable experimental features? (e.g: es-public-fields)',
      default: false,
    });

    // Create new directory
    prompts.push({
      type: 'confirm',
      name: 'gitInit',
      message: 'Do you want to initialize git (init, initial commit & switch to develop)?',
      default: true,
    });

    return this.prompt(prompts).then(props => {
      // To access props later use this.props.someAnswer;
      this.props = props;

      // Replace destionationRoot if createDir
      if (this.props.createDir) {
        this.destinationRoot(this.destinationPath(slug(this.props.name)));
      }
    });
  }

  writing() {
    const defaultTemplateFiles = [
      '.eslintrc',
      '.gitignore',
      '.editorconfig',
      'postcss.config.js',
      'webpack.config.dev.js',
      'webpack.config.prod.js',
      'src',
    ];

    // Other files
    defaultTemplateFiles.forEach((templateFile) => {
      this.fs.copy(
        this.templatePath(templateFile),
        this.destinationPath(templateFile)
      );
    });

    // .babelrc with optional experimental features
    this.fs.copyTpl(
      this.templatePath('.babelrc'),
      this.destinationPath('.babelrc'),
      { experimental: this.props.experimental }
    );

    // package.json (needs variable injection)
    this.fs.copyTpl(
      this.templatePath('package.json'),
      this.destinationPath('package.json'),
      {
        name: this.props.name,
        desc: this.props.desc,
        repo: this.props.repo,
        testing: this.props.testing,
        experimental: this.props.experimental,
      }
    );

    // README (needs variable injection)
    this.fs.copyTpl(
      this.templatePath('README.md'),
      this.destinationPath('README.md'),
      {
        name: this.props.name,
        desc: this.props.desc,
        experimental: this.props.experimental,
      }
    );

    // Copy sample tests
    this.fs.copy(
      this.templatePath(`__tests__/${this.props.testing}`),
      this.destinationPath('__tests__')
    );
  }

  install() {
    // npm install
    this.log('Installing npm packages');
    this.spawnCommandSync('npm', ['install']);

    // Testing: enzyme
    if (this.props.testing === 'jest+enzyme') {
      this.spawnCommandSync('npm', ['install', 'enzyme', 'enzyme-adapter-react-16', '--save-dev']);
    }

    // git
    if (this.props.gitInit) {
      this.log('Initializing git');
      this.spawnCommandSync('git', ['init']);
      this.spawnCommandSync('git', ['add', '.']);
      this.spawnCommandSync('git', ['commit', '-m', 'initial commit', '--quiet']);
      this.spawnCommandSync('git', ['branch', 'develop']);
      this.spawnCommandSync('git', ['checkout', 'develop']);
    }
  }
};
